#include <iostream>

const int dim = 3;
void func1(int mat[][dim]);
void func2(int (*mat)[dim]);
void func3(const int *mat);
void func4(int **mat);
void func5(int *mat[dim]);

int main() {
	int mat[dim][dim];
	for(int i = 0; i < dim; i++) {
		for(int j = 0; j < dim; j++) {
			mat[i][j] = i*10 + j;
		}
	}
	printf("\n");
	printf(" Initialized data to:\n");
	for(auto& i : mat) {
		for(int j : i) {
			printf("%5.2d", j);
		}
		printf("\n");
	}
	func1(mat);
	func2(mat);
	func3((int*)mat);
	func4((int**)mat);
	func5((int**)mat);

	return EXIT_SUCCESS;
}

/* 
 *	Method #1
 *	You don't have to specify the first dimension!
 */

void func1(int mat[][dim]) {
	printf("\n");
	printf(" Declare as matrix, explicitly specify second dimension: ");
	for(int i = 0; i < dim; i++) {
		printf("\n");
		for(int j = 0; j < dim; j++) {
			printf("%5.2d", mat[i][j]);
		}
	}
	printf("\n");
}

/*
 *	Method #2
 *	Pointer to array, second dimension is explicitly specified.
 *	The declaration int (*mat)[dim] says that mat is a pointer
 *	to an array of 'dim' ints.
 */

void func2(int (*mat)[dim]) {
	printf("\n");
	printf(" Declare as pointer to column, explicitly specify 2nd dim: ");
	for(int i = 0; i < dim; i++) {
		printf("\n");
		for(int j = 0; j < dim; j++) {
			printf("%5.2d", mat[i][j]);
		}
	}
	printf("\n");
}

/*
 *	Method #3
 *	Using a single pointer, the array is "flattened"
 *	With this method you can create general-purpose routines.
 *	The dimensions don't appear in any declaration, so you
 *	can add them to the formal argument list. 
 *
 *	The manual array indexing will probably slow down execution.
 */

void func3(const int *mat)	{
	printf("\n");
	printf(" Declare as single-pointer, manual offset computation: ");
	for(int i = 0; i < dim; i++) {
		printf("\n");
		for(int j = 0; j < dim; j++) {
			printf("%5.2d", *(mat + dim*i + j));
		}
	}
	printf("\n");
}

/*
 *	Method #4 (double pointer, using an auxiliary array of pointers).
 *	With this method you can create general-purpose routines,
 *	if you allocate "index" at run-time. 
 *
 *	Add the dimensions to the formal argument list.
 */

void func4(int **mat) {
	printf("\n");
	int* index[dim];	//	index[i] contains address of the ith row
	for (int i = 0; i < dim; i++)
		index[i] = (int*)mat + dim*i;
	printf(" Declare as double-pointer, use auxiliary pointer array: ");
	for(int i = 0; i < dim; i++) {
		printf("\n");
		for(int j = 0; j < dim; j++) {
			printf("%5.2d", index[i][j]);
		}
	}
	printf("\n");
}

/*
 *	Method #5 (single pointer, using an auxiliary array of pointers)
 */

void func5(int *mat[dim]) {
	printf("\n");
	int *index[dim];
	for (int i = 0; i < dim; i++)
		index[i] = (int*)mat + dim*i;
	printf(" Declare as single-pointer, use auxiliary pointer array: ");
	for(int i = 0; i < dim; i++) {
		printf("\n");
		for(int j = 0; j < dim; j++) {
			printf("%5.2d", index[i][j]);
		}
	}
	printf("\n");
}

