#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>

void print_storage_info(const int* next, const int* prev, int ints) {
	if (next)
		printf("%s location: %p. Size: %d ints (%ld bytes).\n",
		       (next != prev ? "New" : "Old"), (void*)next, ints, ints * sizeof(int));
	else
		printf("Allocation failed.\n");
}

int main()
{
	const int pattern[] = {1, 2, 3, 4, 5, 6, 7, 8};
	const int pattern_size = sizeof pattern / sizeof(int);
	int *next, *prev = nullptr;

	if ((next = (int*)malloc(pattern_size * sizeof *next))) {
		memcpy(next, pattern, sizeof pattern); // fills the array
		print_storage_info(next, prev, pattern_size);
	}
	else
		return EXIT_FAILURE;

	// Reallocate in cycle using the following values as a new storage size.
	const int realloc_size[] = {10, 12, 512, 32768, 65536, 32768};

	for (int i : realloc_size) {
		prev = next;
		if ((next = (int*)realloc(prev, i * sizeof(int)))){
			print_storage_info(next, prev, i); // Should not use prev here - undefined behavior
			assert(!memcmp(next, pattern, sizeof pattern));  // is pattern held
		} else {
			free(prev);
			return EXIT_FAILURE;
		}
	}

	free(next); // finally, frees the storage
	return EXIT_SUCCESS;
}

